@@ -37,7 +37,7 @@ macro_rules! info_table_row {
 macro_rules! _info_table_row {
     ($fn: ident, $($element: expr),*) => {
         unsafe {
-            $crate::bindings::$fn($crate::_info_table_row!(@COUNT; $($element),*) as i32, $($crate::functions::c_str($element)),*);
+            $crate::bindings::$fn($crate::_info_table_row!(@COUNT; $($element),*) as i32, $(::std::ffi::CString::new($element).unwrap().as_ptr()),*);
         }
     };
 
@@ -57,6 +57,10 @@ macro_rules! _info_table_row {
 /// [`Zval`](crate::php::types::zval::Zval).
 #[macro_export]
 macro_rules! call_user_func {
+    ($fn: expr) => {
+        $fn.try_call(vec![])
+    };
+
     ($fn: expr, $($param: expr),*) => {
         $fn.try_call(vec![$(&$param),*])
     };
