@@ -1,17 +1,17 @@
 //! Builder and objects for creating classes in the PHP world.
 
 use crate::errors::{Error, Result};
-use std::{convert::TryInto, ffi::CString, fmt::Debug, mem};
+use std::{alloc::Layout, convert::TryInto, ffi::CString, fmt::Debug};
 
 use crate::bindings::{
     zend_class_entry, zend_declare_class_constant, zend_declare_property,
-    zend_register_internal_class_ex,
+    zend_do_implement_interface, zend_register_internal_class_ex,
 };
 
 use super::{
-    executor::ExecutorGlobals,
     flags::{ClassFlags, MethodFlags, PropertyFlags},
     function::FunctionEntry,
+    globals::ExecutorGlobals,
     types::{
         object::{ZendObject, ZendObjectOverride},
         string::ZendString,
@@ -48,6 +48,11 @@ impl ClassEntry {
         ClassFlags::from_bits_truncate(self.ce_flags)
     }
 
+    /// Returns `true` if the class entry is an interface, and `false` otherwise.
+    pub fn is_interface(&self) -> bool {
+        self.flags().contains(ClassFlags::Interface)
+    }
+
     /// Checks if the class is an instance of another class or interface.
     ///
     /// # Parameters
@@ -119,6 +124,7 @@ pub struct ClassBuilder<'a> {
     name: String,
     ptr: &'a mut ClassEntry,
     extends: Option<&'static ClassEntry>,
+    interfaces: Vec<&'static ClassEntry>,
     methods: Vec<FunctionEntry>,
     object_override: Option<unsafe extern "C" fn(class_type: *mut ClassEntry) -> *mut ZendObject>,
     properties: Vec<(String, Zval, PropertyFlags)>,
@@ -138,7 +144,7 @@ impl<'a> ClassBuilder<'a> {
         // which will cause the program to panic (standard in Rust). Unwrapping is OK - the ptr
         // will either be valid or null.
         let ptr = unsafe {
-            (libc::calloc(1, mem::size_of::<ClassEntry>()) as *mut ClassEntry)
+            (std::alloc::alloc_zeroed(Layout::new::<ClassEntry>()) as *mut ClassEntry)
                 .as_mut()
                 .unwrap()
         };
@@ -147,6 +153,7 @@ impl<'a> ClassBuilder<'a> {
             name: name.into(),
             ptr,
             extends: None,
+            interfaces: vec![],
             methods: vec![],
             object_override: None,
             properties: vec![],
@@ -164,6 +171,24 @@ impl<'a> ClassBuilder<'a> {
         self
     }
 
+    /// Implements an interface on the class.
+    ///
+    /// # Parameters
+    ///
+    /// * `interface` - Interface to implement on the class.
+    ///
+    /// # Panics
+    ///
+    /// Panics when the given class entry `interface` is not an interface.
+    pub fn implements(mut self, interface: &'static ClassEntry) -> Self {
+        if !interface.is_interface() {
+            panic!("Given class entry was not an interface.");
+        }
+
+        self.interfaces.push(interface);
+        self
+    }
+
     /// Adds a method to the class.
     ///
     /// # Parameters
@@ -230,8 +255,7 @@ impl<'a> ClassBuilder<'a> {
     /// # Parameters
     ///
     /// * `T` - The type which will override the Zend object. Must implement [`ZendObjectOverride`]
-    /// which can be derived through the [`ZendObjectHandler`](ext_php_rs_derive::ZendObjectHandler)
-    /// derive macro.
+    /// which can be derived using the [`php_class`](crate::php_class) attribute macro.
     pub fn object_override<T: ZendObjectOverride>(mut self) -> Self {
         self.object_override = Some(T::create_object);
         self
@@ -262,7 +286,13 @@ impl<'a> ClassBuilder<'a> {
         };
 
         // SAFETY: We allocated memory for this pointer in `new`, so it is our job to free it when the builder has finished.
-        unsafe { libc::free((self.ptr as *mut ClassEntry) as *mut libc::c_void) };
+        unsafe {
+            std::alloc::dealloc((self.ptr as *mut _) as *mut u8, Layout::new::<ClassEntry>())
+        };
+
+        for iface in self.interfaces {
+            unsafe { zend_do_implement_interface(class, std::mem::transmute(iface)) };
+        }
 
         for (name, mut default, flags) in self.properties {
             unsafe {
