@@ -6,9 +6,9 @@ pub mod constants;
 pub mod enums;
 pub mod exceptions;
 pub mod execution_data;
-pub mod executor;
 pub mod flags;
 pub mod function;
+pub mod globals;
 pub mod module;
 pub mod pack;
 pub mod types;
